/*
 * $Id: descriptor.h,v 1.3 2010-05-22 11:16:39 laci Exp $
 *
 * Project      : RNA motif searching in genomic sequences
 * Description  : the header file for descriptor.cpp
 *
 * Author       : Ladislav Rampasek <rampasek@gmail.com>
 * Institution  : Comenius University in Bratislava
 *
 */

#include "types.h"

// the class of an descriptor
class Descriptor{
    public:
        vector<SSE> sses;           // the array where all SSEs are stored, indexed from 1 (0-th position is grabage)
        vector<int> motif;          // the list of all SSEs, i-th number is index to the array "sses",
                                    //   if the number is negative it signs for the second strand
                                    //   of the helix which has index "-number" in "sses"
        map<string, int> transl;    // a transl. dictionary from original names (h1,s5,...) to indices in "sses"
        vector<int> srch_order;     // the ordered list of SSEs to be searched (indices to the vector "sses")

        Descriptor():initialized(false){};
        Descriptor(ifstream &fin);
        bool is_initialized(){ return initialized;}
        string error_str(){ return err_str;}
        string search_order_to_str();
        int get_max_motif_length();

    private:
        bool initialized;
        string err_str;
        bool parse_desc_map(string line);
        bool parse_desc_properties(string line);
        bool parse_desc_order(string line);
        bool check_consistency();
        bool has_no_duplicates(vector<int> vec);
        void complete_srch_order();
        void compute_sizes();
};
