/*
 * $Id: generalfuncs.cpp,v 1.5 2010-05-22 18:32:47 laci Exp $
 *
 * Project      : RNA motif searching in genomic sequences
 * Description  : auxiliary functions
 *
 * Author       : Ladislav Rampasek <rampasek@gmail.com>
 * Institution  : Comenius University in Bratislava
 *
 */

#include <iostream>
#include <cstdlib>
#include <fstream>
#include <string>
using namespace std;

// end program with an error message
int die(string errmsg){
  cerr<<"ERROR: "<<errmsg<<"\n";
  //cerr<<"The program finished with an error.\n";
  exit(1);
}

// get next noncomment nonempty line from given ifstream
bool get_valuable_line(ifstream &fin, string &str){
    string line;
    string::iterator it;
    while(getline(fin, line)){
        it=line.begin();
        while(isspace(*it)) ++it;

        if( it==line.end() ) continue;

        if( *it!='#' ) {
            str = line;
            return true;
        }
    }

    str = "";
    return false;
}

// evaluate if @ch fits @patt according to IUPAC Notation
bool fits(char ch, char patt){
    ch=toupper(ch);
    patt=toupper(patt);

    if(ch=='U') ch='T';

    if(patt=='*' || ch=='N' || ch==patt) return true;
    switch (patt){
        /*case 'A': //Adenine
            return ch=='A';
        case 'C': //Cytosine
            return ch=='C';
        case 'G': //Guanine
            return ch=='G';*/
        case 'U':
        case 'T': //Thymine in DNA; uracil in RNA
            return ch=='T';
        case 'M': //aMino
            return ch=='A' || ch=='C';
        case 'R': //puRine
            return ch=='A' || ch=='G';
        case 'W': //Weak (2 H bonds)
            return ch=='A' || ch=='T';
        case 'S': //Strong (3 H bonds)
            return ch=='C' || ch=='G';
        case 'Y': //pYrimidine
            return ch=='C' || ch=='T';
        case 'K': //Keto
            return ch=='G' || ch=='T';
        case 'V': //not T
            return ch!='T';
        case 'H': //not G
            return ch!='G';
        case 'D': //not C
            return ch!='C';
        case 'B': //not A
            return ch!='A';
        case 'N': //aNy
            return true;
    }
    return false;
}

// evaluate if @ch_strand1 is complementary to @ch_strand2 according to @transf_matrix and IUPAC Notation
bool is_complemntary(char ch_strand1, char ch_strand2, string &transf_matrix){
    ch_strand1=toupper(ch_strand1);
    ch_strand2=toupper(ch_strand2);

    if(ch_strand1=='*' && ch_strand2=='*') return true; //if both are wildcards - ok
    if(ch_strand1=='*' || ch_strand2=='*') return false; //if only one is wildcard - bad
    if(ch_strand1=='N' || ch_strand2=='N') return true;

    switch (ch_strand1){
        case 'A': //Adenine
            return fits(ch_strand2, transf_matrix[0]);
        case 'C': //Cytosine
            return fits(ch_strand2, transf_matrix[1]);
        case 'G': //Guanine
            return fits(ch_strand2, transf_matrix[2]);
        case 'U':
        case 'T': //Thymine in DNA; uracil in RNA
            return fits(ch_strand2, transf_matrix[3]);
    }
    return true;
}
