/*
 * $Id: matrix.cpp,v 1.1 2010-05-02 14:16:34 laci Exp $
 *
 * Project      : RNA motif searching in genomic sequences
 * Description  : the data structure for dynamic programming
 *
 * Author       : Ladislav Rampasek <rampasek@gmail.com>
 * Institution  : Comenius University in Bratislava
 *
 */

#include <map>
#include <list>
#include <cstdarg>
using namespace std;

//data structure for dynamic programming
class Matrix{
    public:
        Matrix(){Matrix(1);};
        Matrix(unsigned int num_dimensions):num_dimensions(num_dimensions){};

        bool get(int num_args, ...);
        bool get(list<int> &key);
        void set(bool value, int num_args, ...);
        void set(bool value, list<int> &key);
        void clear(){ matrix.clear(); };
        bool empty(){ return matrix.empty(); };
        void set_dimensions(int new_num_dimensions){ num_dimensions=new_num_dimensions; clear(); };
    private:
        unsigned int num_dimensions;
        map<list<int>, bool> matrix;
};

//get value of a cell specified by arguments passed by "..."
bool Matrix::get(int num_args, ...){
    if(num_args!=num_dimensions) return 0;

    list<int> key;

    //get arguments
    va_list argp;
    va_start(argp,num_args);
        for(int i=0;i<num_args;++i) key.push_back( va_arg(argp, int) );
    va_end(argp);

    map<list<int>, bool>::iterator it=matrix.find(key);
    if(it==matrix.end()) return 0; //if this cell doesn't exist, return 0
    else return (it->second);
}

//get value of a cell specified by given key of type list<int>
bool Matrix::get(list<int> &key){
    if(key.size()!=num_dimensions) return 0;

    map<list<int>, bool>::iterator it=matrix.find(key);
    if(it==matrix.end()) return 0; //if this cell doesn't exist, return 0
    else return (it->second);
}

//set value of a cell specified by arguments passed by "..."
void Matrix::set(bool value, int num_args, ...){
    if(num_args!=num_dimensions) return;

    list<int> key;

    //get arguments
    va_list argp;
    va_start(argp,num_args);
        for(int i=0;i<num_args;++i) key.push_back( va_arg(argp, int) );
    va_end(argp);

    matrix[key]=value;
}

//set value of a cell specified by given key of type list<int>
void Matrix::set(bool value, list<int> &key){
    if(key.size()!=num_dimensions) return;

    matrix[key]=value;
}
